﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data.Services.Client;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using FilterTest.Netflix;

namespace FilterTest
{
    public partial class App : Application
    {
        private bool mbGenresIsLoading;
        private bool mbTitlesIsLoading;

        private NetflixCatalog mCatalog;
        private DataServiceCollection<Genre> mGenres;
        private DataServiceCollection<Title> mTitles;

        public NetflixCatalog Catalog
        {
            get
            {
                return mCatalog;
            }
        }

        public DataServiceCollection<Genre> Genres
        {
            get
            {
                return mGenres;
            }
        }

        public bool GenresIsLoading
        {
            get
            {
                return mbGenresIsLoading;
            }
        }

        public DataServiceCollection<Title> Titles
        {
            get
            {
                return mTitles;
            }
        }

        public bool TitlesIsLoading
        {
            get
            {
                return mbTitlesIsLoading;
            }
        }

        public App()
        {
            mCatalog = new NetflixCatalog(new Uri("http://odata.netflix.com/v2/Catalog/", UriKind.Absolute));

            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            worker.RunWorkerAsync();

            this.Startup += this.Application_Startup;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            this.RootVisual = new MainPage();
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            mGenres = new DataServiceCollection<Genre>(); mbGenresIsLoading = true;
            mGenres.LoadCompleted += new EventHandler<LoadCompletedEventArgs>(mGenres_LoadCompleted);
            mGenres.LoadAsync(from g in mCatalog.Genres select g);

            mTitles = new DataServiceCollection<Title>(); mbTitlesIsLoading = true;
            mTitles.LoadCompleted += new EventHandler<LoadCompletedEventArgs>(mTitles_LoadCompleted);
            mTitles.LoadAsync(from t in mCatalog.Titles.Expand("Cast,Genres") where t.Name.Contains("National") select t);
        }

        void mGenres_LoadCompleted(object sender, LoadCompletedEventArgs e)
        {
            DataServiceCollection<Genre> gc = (DataServiceCollection<Genre>)sender;

            if (e.Error == null)
            {
                if (gc.Continuation != null)
                {
                    gc.LoadNextPartialSetAsync();
                }
                else
                {
                    mbGenresIsLoading = false;
                }
            }
            else
            {
                MessageBox.Show(e.Error.Message);
            }
        }

        void mTitles_LoadCompleted(object sender, LoadCompletedEventArgs e)
        {
            DataServiceCollection<Title> tc = (DataServiceCollection<Title>)sender;

            if (e.Error == null)
            {
                if (tc.Continuation != null)
                {
                    tc.LoadNextPartialSetAsync();
                }
                else
                {
                    mbTitlesIsLoading = false;
                }
            }
            else
            {
                MessageBox.Show(e.Error.Message);
            }
        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // a ChildWindow control.
            if (!System.Diagnostics.Debugger.IsAttached)
            {
                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                ChildWindow errorWin = new ErrorWindow(e.ExceptionObject);
                errorWin.Show();
            }
        }
    }
}